/*
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001, 2003
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

#ifndef IVADMINAPI_H
#define IVADMINAPI_H

#if defined(WIN32) || defined(_WIN32)
  #define IVADMIN_CALLTYPE       __cdecl
  #if defined(IVADMINDLL)
    #define IVADMIN_DECLSPEC       __declspec( dllexport )
  #else
    #if defined(IVADMIN_LINK_STATIC)
      #define IVADMIN_DECLSPEC
    #else
      #define IVADMIN_DECLSPEC       __declspec( dllimport )
    #endif /* IVADMIN_LINK_STATIC */
  #endif /* IVADMINDLL */
#else /* WIN32 */
  #define IVADMIN_CALLTYPE
  #define IVADMIN_DECLSPEC
#endif /* WIN32 */

/*
   This now pre-reqs the AZN ADK.
*/
#include <ogauthzn.h>

/* boolean */
#define IVADMIN_FALSE     0
#define IVADMIN_TRUE      1

#define IVADMIN_AUDIT_NONE (0)
#define IVADMIN_AUDIT_PERMIT (1)
#define IVADMIN_AUDIT_DENY (2)
#define IVADMIN_AUDIT_ERROR (4)
#define IVADMIN_AUDIT_ADMIN (8)
#define IVADMIN_AUDIT_ALL (15)

#define IVADMIN_TIME_LOCAL (0)
#define IVADMIN_TIME_UTC (1)
#define IVADMIN_TOD_ANY (0)
#define IVADMIN_TOD_SUN (1)
#define IVADMIN_TOD_MON (2)
#define IVADMIN_TOD_TUE (4)
#define IVADMIN_TOD_WED (8)
#define IVADMIN_TOD_THU (16)
#define IVADMIN_TOD_FRI (32)
#define IVADMIN_TOD_SAT (64)
#define IVADMIN_TOD_ALL (127)
#define IVADMIN_TOD_WEEKDAY (62)
#define IVADMIN_TOD_WEEKEND (65)
#define IVADMIN_TOD_MINUTES (60)
#define IVADMIN_TOD_OCLOCK (3600)

/*
 * Use the following two identifiers in APIs that take a domain
 * name (except ivadmin_domain_create).  Use IVADMIN_DOMAIN_MANAGEMENT
 * to specify the installation's management domain and use
 * IVADMIN_DOMAIN_LOCAL to specify the local domain configured for the
 * current runtime.  A domain cannot be created with either of these
 * reserved names.
 */
#define IVADMIN_DOMAIN_MANAGEMENT     "/DomainManagement/"
#define IVADMIN_DOMAIN_LOCAL          "/DomainLocal/"

/*
 * Use the following identifiers in ivadmin_context_create3 to specify a
 * locale that returned data should be translated into: UTF-8 or the codeset
 * defined in the operating environment of the local machine.
 */
#define IVADMIN_CODESET_UTF8    "UTF8"
#define IVADMIN_CODESET_LOCAL   "local"

/*
 * The following #defines are to be used for the
 * ivadmin_cfg_getvalue() API.
 */

/*  Access Manager configuration files */
#define IVADMIN_CFG_CFGFILE_AMRTE           "pd.conf"
#define IVADMIN_CFG_CFGFILE_AMMGRD          "ivmgrd.conf"
#define IVADMIN_CFG_CFGFILE_AMACLD          "ivacld.conf"
#define IVADMIN_CFG_CFGFILE_AMPROXY         "pdproxy.conf"

/*  Access Manager stanzas */
#define IVADMIN_CFG_STANZA_SSL              "ssl"
#define IVADMIN_CFG_STANZA_POLICY_SVR       "manager"
#define IVADMIN_CFG_STANZA_LDAP             "ldap"

/*  Access Manager keys */
#define IVADMIN_CFG_KEY_POLICY_SVR          "new-policy-hostport"
#define IVADMIN_CFG_KEY_POLICY_SVR_HOSTNAME "master-host"
#define IVADMIN_CFG_KEY_POLICY_SVR_PORT     "master-port"
#define IVADMIN_CFG_KEY_SSL_KEYFILE         "ssl-keyfile"
#define IVADMIN_CFG_KEY_SSL_STASHFILE       "ssl-keyfile-stash"
#define IVADMIN_CFG_KEY_SSL_PASSWORD        "ssl-keyfile-pwd"
#define IVADMIN_CFG_KEY_SSL_LOCAL_DOMAIN    "ssl-local-domain"
#define IVADMIN_CFG_KEY_REGISTRY_PASSWORD   "bind-pwd"


#ifdef __cplusplus
    extern "C" {
#endif

typedef struct sivadmin_context* ivadmin_context;
typedef struct sivadmin_response* ivadmin_response;
typedef struct sivadmin_ldapgroup* ivadmin_ldapgroup;
typedef struct sivadmin_ldapuser* ivadmin_ldapuser;
typedef struct sivadmin_protobj* ivadmin_protobj;
typedef struct sivadmin_acl* ivadmin_acl;
typedef struct sivadmin_action* ivadmin_action;
typedef struct sivadmin_ssoweb* ivadmin_ssoweb;
typedef struct sivadmin_ssogroup* ivadmin_ssogroup;
typedef struct sivadmin_ssocred* ivadmin_ssocred;
typedef struct sivadmin_pop* ivadmin_pop;
typedef struct sivadmin_domain* ivadmin_domain;
typedef struct sivadmin_authzrule* ivadmin_authzrule;
typedef struct sivadmin_accessOutdata* ivadmin_accessOutdata;

typedef enum {authz=1, policy} ivadmin_cfg_replicatype;

IVADMIN_DECLSPEC void IVADMIN_CALLTYPE
   ivadmin_free(void *p);

/******************************************************
 *
 * Context APIs
 *
 ******************************************************/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_create3(const char *userid,
                           const char *pwd,
                           const char *domain,
                           const char *codeset,
                           const char *serverhost,
                           unsigned long port,
                           const char *keyringfile,
                           const char *keyringstashfile,
                           const char *configfile,
                           ivadmin_context *ctx,
                           ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_createdefault2(const char *userid,
                                  const char *pwd,
                                  const char *domainid,
                                  ivadmin_context *ctx,
                                  ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_createlocal(const char *userid,
                               const char *pwd,
                               const char *domainid,
                               const char *codeset,
                               ivadmin_context *ctx,
                               ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_delete(ivadmin_context ctx,
                          ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setdelcred(ivadmin_context ctx,
                              const unsigned char* pacValue,
                              const unsigned long pacLength,
                              ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_cleardelcred(ivadmin_context ctx,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_context_getdomainid(ivadmin_context ctx);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_context_getmgmtdomainid(ivadmin_context ctx);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_context_getuserid(ivadmin_context ctx);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_context_getcodeset(ivadmin_context ctx);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_domainismanagement(ivadmin_context ctx);

#define IVADMIN_CONTEXT_DCEUSERREG  0
#define IVADMIN_CONTEXT_LDAPUSERREG 1
#define IVADMIN_CONTEXT_ADUSERREG 2
#define IVADMIN_CONTEXT_DOMINOUSERREG 3
#define IVADMIN_CONTEXT_MULTIDOMAIN_ADUSERREG 4

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getuserreg(ivadmin_context ctx,
                              unsigned long *registry,
                              ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setminpwdlen(ivadmin_context ctx,
                                unsigned long length,
                                unsigned long unset,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getminpwdlen(ivadmin_context ctx,
                                unsigned long *length,
                                unsigned long *unset,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setmaxpwdage(ivadmin_context ctx,
                                unsigned long seconds,
                                unsigned long unset,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getmaxpwdage(ivadmin_context ctx,
                                unsigned long *seconds,
                                unsigned long *unset,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setaccexpdate(ivadmin_context ctx,
                                 unsigned long seconds,
                                 unsigned long unlimited,
                                 unsigned long unset,
                                 ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getaccexpdate(ivadmin_context ctx,
                                 unsigned long *seconds,
                                 unsigned long *unlimited,
                                 unsigned long *unset,
                                 ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setmaxlgnfails(ivadmin_context ctx,
                                  unsigned long failures,
                                  unsigned long unset,
                                  ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getmaxlgnfails(ivadmin_context ctx,
                                  unsigned long *failures,
                                  unsigned long *unset,
                                  ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setdisabletimeint(ivadmin_context ctx,
                                     unsigned long seconds,
                                     unsigned long disable,
                                     unsigned long unset,
                                     ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getdisabletimeint(ivadmin_context ctx,
                                     unsigned long *seconds,
                                     unsigned long *disable,
                                     unsigned long *unset,
                                     ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setmaxpwdrepchars(ivadmin_context ctx,
                                     unsigned long chars,
                                     unsigned long unset,
                                     ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getmaxpwdrepchars(ivadmin_context ctx,
                                     unsigned long *chars,
                                     unsigned long *unset,
                                     ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setminpwdalphas(ivadmin_context ctx,
                                   unsigned long chars,
                                   unsigned long unset,
                                   ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getminpwdalphas(ivadmin_context ctx,
                                   unsigned long *chars,
                                   unsigned long *unset,
                                   ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setminpwdnonalphas(ivadmin_context ctx,
                                      unsigned long chars,
                                      unsigned long unset,
                                      ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getminpwdnonalphas(ivadmin_context ctx,
                                      unsigned long *chars,
                                      unsigned long *unset,
                                      ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_setpwdspaces(ivadmin_context ctx,
                                unsigned long allowed,
                                unsigned long unset,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_getpwdspaces(ivadmin_context ctx,
                                unsigned long *allowed,
                                unsigned long *unset,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_settodaccess(ivadmin_context ctx,
                                unsigned long days,
                                unsigned long start,
                                unsigned long end,
                                unsigned long reference,
                                unsigned long unset,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_context_gettodaccess(ivadmin_context ctx,
                                unsigned long *days,
                                unsigned long *start,
                                unsigned long *end,
                                unsigned long *reference,
                                unsigned long *unset,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC const char *  IVADMIN_CALLTYPE
   ivadmin_context_getmgmtsvrhost(ivadmin_context ctx);

IVADMIN_DECLSPEC unsigned long  IVADMIN_CALLTYPE
   ivadmin_context_getmgmtsvrport(ivadmin_context ctx);

/******************************************************
 *
 * User APIs
 *
 ******************************************************/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_create3(ivadmin_context ctx,
                        const char *userid,
                        const char *dn,
                        const char *cn,
                        const char *sn,
                        const char *pwd,
                        unsigned long group_count,
                        const char **groups,
                        unsigned long ssouser,
                        unsigned long nopwdpolicy,
                        ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_import2(ivadmin_context ctx,
                        const char *userid,
                        const char *dn,
                        const char *groupid,
                        unsigned long ssouser,
                        ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setdescription(ivadmin_context ctx,
                               const char *userid,
                               const char *description,
                               ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setpassword(ivadmin_context ctx,
                            const char *userid,
                            const char *pwd,
                            ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setaccountvalid(ivadmin_context ctx,
                                const char *userid,
                                unsigned long valid,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setpasswordvalid(ivadmin_context ctx,
                                 const char *userid,
                                 unsigned long valid,
                                 ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setssouser(ivadmin_context ctx,
                           const char *userid,
                           unsigned long ssouser,
                           ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_delete2(ivadmin_context ctx,
                        const char *userid,
                        unsigned long registry,
                        ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getmemberships(ivadmin_context ctx,
                               const char *userid,
                               unsigned long *count,
                               char ***groupids,
                               ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_get(ivadmin_context ctx,
                    const char *userid,
                    ivadmin_ldapuser *user,
                    ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getbydn(ivadmin_context ctx,
                        const char *dn,
                        ivadmin_ldapuser *user,
                        ivadmin_response *rsp);

#define IVADMIN_MAXRETURN 0
#define IVADMIN_ALLPATTERN "*"

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_list(ivadmin_context ctx,
                     const char *pattern,
                     unsigned long maxreturn,
                     unsigned long *count,
                     char ***userids,
                     ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_listbydn(ivadmin_context ctx,
                         const char *pattern,
                         unsigned long maxreturn,
                         unsigned long *count,
                         char ***dns,
                         ivadmin_response *rsp);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_user_getid(ivadmin_ldapuser user);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_user_getdescription(ivadmin_ldapuser user);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getaccountvalid(ivadmin_ldapuser user);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getpasswordvalid(ivadmin_ldapuser user);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getssouser(ivadmin_ldapuser user);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_user_getdn(ivadmin_ldapuser user);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_user_getcn(ivadmin_ldapuser user);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_user_getsn(ivadmin_ldapuser user);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setaccexpdate(ivadmin_context ctx,
                              const char *userid,
                              unsigned long seconds,
                              unsigned long unlimited,
                              unsigned long unset,
                              ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getaccexpdate(ivadmin_context ctx,
                              const char *userid,
                              unsigned long *seconds,
                              unsigned long *unlimited,
                              unsigned long *unset,
                              ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_settodaccess(ivadmin_context ctx,
                             const char *userid,
                             unsigned long days,
                             unsigned long start,
                             unsigned long end,
                             unsigned long reference,
                                unsigned long unset,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_gettodaccess(ivadmin_context ctx,
                             const char *userid,
                             unsigned long *days,
                             unsigned long *start,
                             unsigned long *end,
                             unsigned long *reference,
                                unsigned long *unset,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setminpwdlen(ivadmin_context ctx,
                             const char *userid,
                             unsigned long length,
                             unsigned long unset,
                             ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getminpwdlen(ivadmin_context ctx,
                             const char *userid,
                             unsigned long *length,
                             unsigned long *unset,
                             ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setmaxpwdage(ivadmin_context ctx,
                             const char *userid,
                             unsigned long seconds,
                             unsigned long unset,
                             ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getmaxpwdage(ivadmin_context ctx,
                             const char *userid,
                             unsigned long *seconds,
                             unsigned long *unset,
                             ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setmaxlgnfails(ivadmin_context ctx,
                               const char *userid,
                               unsigned long failures,
                               unsigned long unset,
                               ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getmaxlgnfails(ivadmin_context ctx,
                               const char *userid,
                               unsigned long *failures,
                               unsigned long *unset,
                               ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setdisabletimeint(ivadmin_context ctx,
                                  const char *userid,
                                  unsigned long seconds,
                                  unsigned long disable,
                                  unsigned long unset,
                                  ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getdisabletimeint(ivadmin_context ctx,
                                  const char *userid,
                                  unsigned long *seconds,
                                  unsigned long *disable,
                                  unsigned long *unset,
                                  ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setmaxpwdrepchars(ivadmin_context ctx,
                                  const char *userid,
                                  unsigned long chars,
                                  unsigned long unset,
                                  ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getmaxpwdrepchars(ivadmin_context ctx,
                                  const char *userid,
                                  unsigned long *chars,
                                  unsigned long *unset,
                                  ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setminpwdalphas(ivadmin_context ctx,
                                const char *userid,
                                unsigned long chars,
                                unsigned long unset,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getminpwdalphas(ivadmin_context ctx,
                                const char *userid,
                                unsigned long *chars,
                                unsigned long *unset,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setminpwdnonalphas(ivadmin_context ctx,
                                   const char *userid,
                                   unsigned long chars,
                                   unsigned long unset,
                                   ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getminpwdnonalphas(ivadmin_context ctx,
                                   const char *userid,
                                   unsigned long *chars,
                                   unsigned long *unset,
                                   ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_setpwdspaces(ivadmin_context ctx,
                             const char *userid,
                             unsigned long allowed,
                             unsigned long unset,
                             ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_user_getpwdspaces(ivadmin_context ctx,
                             const char *userid,
                             unsigned long *allowed,
                             unsigned long *unset,
                             ivadmin_response *rsp);

/******************************************************
 *
 * Group APIs
 *
 ******************************************************/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_create2(ivadmin_context ctx,
                         const char *groupid,
                         const char *dn,
                         const char *cn,
                         const char *group_container,
                         ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_import2(ivadmin_context ctx,
                         const char *groupid,
                         const char *dn,
                         const char *group_container,
                         ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_setdescription(ivadmin_context ctx,
                                const char *groupid,
                                const char *description,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_addmembers(ivadmin_context ctx,
                           const char *groupid,
                           unsigned long userCount,
                           const char **users,
                           ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_removemembers(ivadmin_context ctx,
                           const char *groupid,
                           unsigned long userCount,
                           const char **users,
                           ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_delete2(ivadmin_context ctx,
                         const char *groupid,
                         unsigned long registry,
                         ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_get(ivadmin_context ctx,
                     const char *groupid,
                     ivadmin_ldapgroup *group,
                     ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_getbydn(ivadmin_context ctx,
                         const char *dn,
                         ivadmin_ldapgroup *group,
                         ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_list(ivadmin_context ctx,
                      const char *pattern,
                      unsigned long maxreturn,
                      unsigned long *count,
                      char ***groupids,
                      ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_listbydn(ivadmin_context ctx,
                          const char *pattern,
                          unsigned long maxreturn,
                          unsigned long *count,
                          char ***dns,
                          ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_group_getmembers(ivadmin_context ctx,
                            const char *groupid,
                            unsigned long *count,
                            char ***userids,
                            ivadmin_response *rsp);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_group_getid(ivadmin_ldapgroup group);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_group_getdescription(ivadmin_ldapgroup group);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_group_getdn(ivadmin_ldapgroup group);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_group_getcn(ivadmin_ldapgroup group);

/******************************************************
 *
 * Objectspace APIs
 *
 ******************************************************/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_objectspace_create(ivadmin_context ctx,
                              const char *objspaceid,
                              unsigned long type,
                              const char *description,
                              ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_objectspace_list(ivadmin_context ctx,
                            unsigned long *count,
                            char ***objspace_list,
                            ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_objectspace_delete(ivadmin_context ctx,
                              const char *objspaceid,
                              ivadmin_response *rsp);

/******************************************************
 *
 * Protected object APIs
 *
 ******************************************************/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_create(ivadmin_context ctx,
                          const char *objid,
                          unsigned long type,
                          const char *description,
                          ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_delete(ivadmin_context ctx,
                          const char *objid,
                          ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_setdesc(ivadmin_context ctx,
                               const char *objid,
                               const char *description,
                               ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_settype(ivadmin_context ctx,
                               const char *objid,
                               unsigned long type,
                               ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_setpolicyattachable(ivadmin_context ctx,
                                           const char *objid,
                                           unsigned long flag,
                                           ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_attrput(ivadmin_context ctx,
                            const char *objid,
                            const char *attr_name,
                            const char *attr_value,
                            ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_attrget(ivadmin_protobj protobj,
                            const char *attr_key,
                            unsigned long *count,
                            char ***attr_value);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_attrdelkey(ivadmin_context ctx,
                               const char *objid,
                               const char *attr_name,
                               ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_attrdelval(ivadmin_context ctx,
                              char *objid,
                              char *attr_key,
                              char *attr_value,
                             ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_attrlist(ivadmin_protobj protobj,
                            unsigned long *count,
                            char ***attrs_list);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_list3(ivadmin_context ctx,
                         const char *objid,
                         azn_attrlist_h_t *indata,
                         unsigned long *objcount,
                         char ***objs,
                         azn_attrlist_h_t *outdata,
                         unsigned long *resultcount,
                         char ***results,
                         ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_listbyacl(ivadmin_context ctx,
                             const char *aclid,
                             unsigned long *count,
                             char ***objids,
                             ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_listbyauthzrule(ivadmin_context ctx,
                             const char *authzruleid,
                             unsigned long *count,
                             char ***objids,
                             ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
ivadmin_protobj_get3(ivadmin_context ctx,
                     const char *objid,
                     azn_attrlist_h_t *indata,
                     ivadmin_protobj *obj,
                     azn_attrlist_h_t *outdata,
                     unsigned long *resultcount,
                     char ***results,
                     ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_attachacl(ivadmin_context ctx,
                             const char *objid,
                             const char *aclid,
                             ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_detachacl(ivadmin_context ctx,
                             const char *objid,
                             ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_attachauthzrule(ivadmin_context ctx,
                             const char *objid,
                             const char *authzruleid,
                             ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_detachauthzrule(ivadmin_context ctx,
                             const char *objid,
                             ivadmin_response *rsp);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_protobj_getid(ivadmin_protobj protobj);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_gettype(ivadmin_protobj protobj);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_protobj_getdesc(ivadmin_protobj protobj);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_protobj_getpolicyattachable(ivadmin_protobj protobj);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_protobj_getaclid(ivadmin_protobj protobj);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_protobj_getpopid(ivadmin_protobj protobj);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_protobj_getauthzruleid(ivadmin_protobj protobj);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_protobj_geteffaclid(ivadmin_protobj protobj);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_protobj_geteffpopid(ivadmin_protobj protobj);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_protobj_geteffauthzruleid(ivadmin_protobj protobj);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
ivadmin_protobj_exists(ivadmin_context ctx,
                       const char *objid,
                       unsigned long *exists,
                       ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
ivadmin_protobj_access(ivadmin_context ctx,
                       char * objid,
                       char *permissionStr,
                       azn_attrlist_h_t *appContext,
                       ivadmin_accessOutdata *outdata,
                       ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
ivadmin_protobj_multiaccess(ivadmin_context ctx,
                       char * objid[],
                       char *permissionStr[],
                       azn_attrlist_h_t *appContext[],
                       ivadmin_accessOutdata *outdata[],
                       int count,
                       ivadmin_response *rsp);

IVADMIN_DECLSPEC ivadmin_response IVADMIN_CALLTYPE
   ivadmin_accessOutdata_getResponseInfo(ivadmin_accessOutdata outdata);

IVADMIN_DECLSPEC unsigned int IVADMIN_CALLTYPE
   ivadmin_accessOutdata_getAccessResult(ivadmin_accessOutdata outdata);

IVADMIN_DECLSPEC azn_attrlist_h_t  IVADMIN_CALLTYPE
   ivadmin_accessOutdata_getPermInfo(ivadmin_accessOutdata outdata);

#define IVADMIN_PROTOBJ_TYPE__UNKNOWN          0
#define IVADMIN_PROTOBJ_TYPE__DOMAIN           1
#define IVADMIN_PROTOBJ_TYPE__FILE             2
#define IVADMIN_PROTOBJ_TYPE__PROGRAM          3
#define IVADMIN_PROTOBJ_TYPE__DIR              4
#define IVADMIN_PROTOBJ_TYPE__JNCT             5
#define IVADMIN_PROTOBJ_TYPE__WEBSEAL_SVR      6
#define IVADMIN_PROTOBJ_TYPE__NETSEAL_SVR      7
#define IVADMIN_PROTOBJ_TYPE__EXTERN_AUTH_SVR  8
#define IVADMIN_PROTOBJ_TYPE__HTTP_SVR         9
#define IVADMIN_PROTOBJ_TYPE__NON_EXIST_OBJ    10
#define IVADMIN_PROTOBJ_TYPE__CONTAINER        11
#define IVADMIN_PROTOBJ_TYPE__LEAF             12
#define IVADMIN_PROTOBJ_TYPE__PORT             13
#define IVADMIN_PROTOBJ_TYPE__APP_CONTAINER    14
#define IVADMIN_PROTOBJ_TYPE__APP_LEAF         15
#define IVADMIN_PROTOBJ_TYPE__MGMT_OBJ         16
#define IVADMIN_PROTOBJ_TYPE__NETSEAL_NET      17

/******************************************************
 *
 * POP APIs
 *
 ******************************************************/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_create(ivadmin_context ctx,
                      const char *popid,
                      ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_delete(ivadmin_context ctx,
                      const char *popid,
                      ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_list(ivadmin_context ctx,
                    unsigned long *count,
                    char ***poplist,
                    ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_attach(ivadmin_context ctx,
                      char *popid,
                      char *objid,
                      ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_detach(ivadmin_context ctx,
                      char *objid,
                      ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_find(ivadmin_context ctx,
                    char *popid,
                    unsigned long *count,
                    char ***obj_list,
                    ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_get(ivadmin_context ctx,
                   char *popid,
                   ivadmin_pop *pop,
                   ivadmin_response *rsp);

IVADMIN_DECLSPEC const char* IVADMIN_CALLTYPE
   ivadmin_pop_getid(ivadmin_pop pop);

IVADMIN_DECLSPEC const char* IVADMIN_CALLTYPE
   ivadmin_pop_getdescription(ivadmin_pop pop);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_setdescription(ivadmin_context ctx,
                              char *popid,
                              char *desc,
                              ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_gettod(ivadmin_pop pop,
                      unsigned long *days,
                      unsigned long *start,
                      unsigned long *end,
                      unsigned long *reference);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_settod(ivadmin_context ctx,
                      char *popid,
                      unsigned long days,
                      unsigned long start,
                      unsigned long end,
                      unsigned long reference,
                      ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_getanyothernw(ivadmin_pop pop,
                             unsigned long *level);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_setanyothernw(ivadmin_context ctx,
                             char *popid,
                             unsigned long level,
                             ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_setanyothernw_forbidden(ivadmin_context ctx,
                                       char *popid,
                                       ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_getipauth(ivadmin_pop pop,
                         unsigned long *count,
                         unsigned long **networks,
                         unsigned long **netmasks,
                         unsigned long **authMethods);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_setipauth(ivadmin_context ctx,
                         char *popid,
                         unsigned long network,
                         unsigned long netmask,
                         unsigned long authMethod,
                         ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_setipauth_forbidden(ivadmin_context ctx,
                                   char *popid,
                                   unsigned long network,
                                   unsigned long netmask,
                                   ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_removeipauth(ivadmin_context ctx,
                            char *popid,
                            char *network,
                            char *netmask,
                            ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_getwarnmode(ivadmin_pop pop);


IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_setwarnmode(ivadmin_context ctx,
                           char *popid,
                           unsigned long warn_mode,
                           ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_getauditlevel(ivadmin_pop pop);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_setauditlevel(ivadmin_context ctx,
                             char *popid,
                             unsigned long audit_level,
                             ivadmin_response *rsp);

IVADMIN_DECLSPEC const char* IVADMIN_CALLTYPE
   ivadmin_pop_getqop(ivadmin_pop pop);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_setqop(ivadmin_context ctx,
                      char *popid,
                      char *qop_level,
                      ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_attrlist(ivadmin_pop pop,
                        unsigned long *count,
                        char ***attr_list);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_attrget(ivadmin_pop pop,
                       char *attr_key,
                       unsigned long *count,
                       char ***attr_value);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_attrput(ivadmin_context ctx,
                       char *popid,
                       char *attr_key,
                       char *attr_value,
                       ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_attrdelkey(ivadmin_context ctx,
                          char *popid,
                          char *attr_key,
                          ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_pop_attrdelval(ivadmin_context ctx,
                          char *popid,
                          char *attr_key,
                          char *attr_value,
                          ivadmin_response *rsp);

/******************************************************
 *
 * Action APIs
 *
 ******************************************************/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_create(ivadmin_context ctx,
                         const char *actionid,
                         const char *description,
                         const char *type,
                         ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_create_in_group(ivadmin_context ctx,
                                  const char *actionid,
                                  const char *description,
                                  const char *type,
                                  const char *groupname,
                                  ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_delete(ivadmin_context ctx,
                         const char *actionid,
                         ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_delete_from_group(ivadmin_context ctx,
                                    const char *actionid,
                                    const char *groupname,
                                    ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_list(ivadmin_context ctx,
                       unsigned long *count,
                       ivadmin_action **actions,
                       ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_list_in_group(ivadmin_context ctx,
                                const char *actiongroup,
                                unsigned long *count,
                                ivadmin_action **actions,
                                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_group_list(ivadmin_context ctx,
                             unsigned long *count,
                             char ***names,
                             ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_group_delete(ivadmin_context ctx,
                               const char *groupname,
                               ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_action_group_create(ivadmin_context ctx,
                               const char *groupname,
                               ivadmin_response *rsp);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_action_getid(ivadmin_action action);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_action_getdescription(ivadmin_action action);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_action_gettype(ivadmin_action action);

/******************************************************
 *
 * ACL APIs
 *
 ******************************************************/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_create(ivadmin_context ctx,
                      const char *aclid,
                      ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_delete(ivadmin_context ctx,
                      const char *aclid,
                      ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_list(ivadmin_context ctx,
                    unsigned long *count,
                    char ***aclids,
                    ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_get(ivadmin_context ctx,
                   const char *aclid,
                   ivadmin_acl *acl,
                   ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_setdescription(ivadmin_context ctx,
                              const char *aclid,
                              const char *description,
                              ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_removeuser(ivadmin_context ctx,
                          const char *aclid,
                          const char *userid,
                          ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_removegroup(ivadmin_context ctx,
                           const char *aclid,
                           const char *groupid,
                           ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_removeanyother(ivadmin_context ctx,
                              const char *aclid,
                              ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_removeunauth(ivadmin_context ctx,
                            const char *aclid,
                            ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_setuser(ivadmin_context ctx,
                       const char *aclid,
                       const char *userid,
                       const char *actions,
                       ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_setgroup(ivadmin_context ctx,
                        const char *aclid,
                        const char *groupid,
                        const char *actions,
                        ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_setanyother(ivadmin_context ctx,
                           const char *aclid,
                           const char *actions,
                           ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_setunauth(ivadmin_context ctx,
                         const char *aclid,
                         const char *actions,
                         ivadmin_response *rsp);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_acl_getid(ivadmin_acl acl);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_acl_getdescription(ivadmin_acl acl);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_listusers(ivadmin_acl acl,
                         unsigned long *count,
                         char ***userids);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_listgroups(ivadmin_acl acl,
                          unsigned long *count,
                          char ***groupids);
IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_acl_getuser(ivadmin_acl acl,
                       const char *userid);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_acl_getgroup(ivadmin_acl acl,
                        const char *groupid);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_acl_getanyother(ivadmin_acl acl);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_acl_getunauth(ivadmin_acl acl);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_attrlist(ivadmin_acl acl,
                        unsigned long *count,
                        char ***attr_list);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_attrget(ivadmin_acl acl,
                       char *attr_key,
                       unsigned long *count,
                       char ***attr_value);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_attrput(ivadmin_context ctx,
                       char *aclid,
                       char *attr_key,
                       char *attr_value,
                       ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_attrdelkey(ivadmin_context ctx,
                          char *aclid,
                          char *attr_key,
                          ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_acl_attrdelval(ivadmin_context ctx,
                          char *aclid,
                          char *attr_key,
                          char *attr_value,
                          ivadmin_response *rsp);

/******************************************************
 *
 * Ssoweb APIs
 *
 ******************************************************/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssoweb_list(ivadmin_context ctx,
                       unsigned long *count,
                       char ***ssowebids,
                       ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssoweb_get(ivadmin_context ctx,
                      const char *ssowebid,
                      ivadmin_ssoweb *ssoweb,
                      ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssoweb_create(ivadmin_context ctx,
                         const char *ssowebid,
                         const char *description,
                         ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssoweb_delete(ivadmin_context ctx,
                         const char *ssowebid,
                         ivadmin_response *rsp);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssoweb_getid(ivadmin_ssoweb ssoweb);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssoweb_getdescription(ivadmin_ssoweb ssoweb);

/******************************************************
 *
 * Ssogroup APIs
 *
 ******************************************************/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssogroup_list(ivadmin_context ctx,
                         unsigned long *count,
                         char ***ssogroupids,
                         ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssogroup_get(ivadmin_context ctx,
                        const char *ssogroupid,
                        ivadmin_ssogroup *ssogroup,
                        ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssogroup_create(ivadmin_context ctx,
                           const char *ssogroupid,
                           const char *description,
                           ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssogroup_delete(ivadmin_context ctx,
                           const char *ssogroupid,
                           ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssogroup_addres(ivadmin_context ctx,
                           const char *ssogroupid,
                           const char *ssoid,
                           ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssogroup_removeres(ivadmin_context ctx,
                              const char *ssogroupid,
                              const char *ssoid,
                              ivadmin_response *rsp);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssogroup_getid(ivadmin_ssogroup ssogroup);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssogroup_getdescription(ivadmin_ssogroup ssogroup);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssogroup_getresources(ivadmin_ssogroup ssogroup,
                                 unsigned long *count,
                                 char ***ssoids);

/******************************************************
 *
 * Ssocred APIs
 *
 ******************************************************/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssocred_list(ivadmin_context ctx,
                        const char *userid,
                        unsigned long *count,
                        ivadmin_ssocred **ssocreds,
                        ivadmin_response *rsp);

#define IVADMIN_SSOCRED_SSOWEB     0
#define IVADMIN_SSOCRED_SSOGROUP   1

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssocred_get(ivadmin_context ctx,
                       const char *ssoid,
                       unsigned long ssotype,
                       const char *userid,
                       ivadmin_ssocred *ssocred,
                       ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssocred_create(ivadmin_context ctx,
                          const char *ssoid,
                          unsigned long ssotype,
                          const char *userid,
                          const char *ssouserid,
                          const char *ssopassword,
                          ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssocred_delete(ivadmin_context ctx,
                          const char *ssoid,
                          unsigned long ssotype,
                          const char *userid,
                          ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssocred_set(ivadmin_context ctx,
                       const char *ssoid,
                       unsigned long ssotype,
                       const char *userid,
                       const char *ssouserid,
                       const char *ssopassword,
                       ivadmin_response *rsp);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssocred_getid(ivadmin_ssocred ssocred);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_ssocred_gettype(ivadmin_ssocred ssocred);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssocred_getuser(ivadmin_ssocred ssocred);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssocred_getssouser(ivadmin_ssocred ssocred);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_ssocred_getssopassword(ivadmin_ssocred ssocred);

/******************************************************
 *
 * Domain APIs
 *
 ******************************************************/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_domain_create(ivadmin_context ctx,
                         const char *domainid,
                         const char *adminid,
                         const char *password,
                         const char *description,
                         ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_domain_delete(ivadmin_context ctx,
                         const char *domainid,
                         unsigned long registry,
                         ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_domain_get(ivadmin_context ctx,
                      const char *domainid,
                      ivadmin_domain *domain,
                      ivadmin_response *rsp);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_domain_getdescription(ivadmin_domain domain);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_domain_getid(ivadmin_domain domain);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_domain_list(ivadmin_context ctx,
                       unsigned long *count,
                       char ***domainids,
                       ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_domain_setdescription(ivadmin_context ctx,
                                 const char *domainid,
                                 const char *description,
                                 ivadmin_response *rsp);

/******************************************************
 *
 * Response APIs
 *
 ******************************************************/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_response_getok(ivadmin_response rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_response_getcount(ivadmin_response rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_response_getcode(ivadmin_response rsp, unsigned long index);

#define IVADMIN_REASON_ALREADY_EXISTS 1

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_response_getreason(ivadmin_response rsp);

#define IVADMIN_RESPONSE_INFO     0
#define IVADMIN_RESPONSE_WARNING  1
#define IVADMIN_RESPONSE_ERROR    2

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_response_getmodifier(ivadmin_response rsp, unsigned long index);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
   ivadmin_response_getmessage(ivadmin_response rsp, unsigned long index);

/******************************************************
 *
 * Cfg APIs
 *
 ******************************************************/
#define ivadmin_cfg_servertype ivadmin_cfg_servermode
typedef enum {local=1,remote} ivadmin_cfg_servermode;

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_cfg_configureserver3(ivadmin_context        ctx,
                               const char              *cfg_file_name,
                               const char              *kdb_dir_name,
                               const char              *application_name,
                               const char              *host_name,
                               ivadmin_cfg_servermode  server_mode,
                               const char              *server_pwd,
                               int                     enable_listening,
                               int                     listening_port,
                               int                     enable_refresh,
                               int                     kdb_pwd_life,
                               int                     ssl_timeout,
                               const char              *appl_cert,
                               unsigned long           group_count,
                               const char              **groups,
                               const char              *description,
                               ivadmin_response        *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_cfg_unconfigureserver(ivadmin_context       ctx,
                                 const char            *cfg_file_name,
                                 const char            *server_name,
                                 const char            *host_name,
                                 ivadmin_response      *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
      ivadmin_cfg_renewservercert2(ivadmin_context      ctx,
                                  const char           *cfg_file_name,
                                  ivadmin_response     *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
      ivadmin_cfg_setport2(ivadmin_context             ctx,
                           const char                  *cfg_file_name,
                           int                         listening_port,
                           ivadmin_response            *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
      ivadmin_cfg_setlistening2(ivadmin_context        ctx,
                                const char             *cfg_file_name,
                                int                    enable_listening,
                                ivadmin_response       *rsp);


IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_cfg_setkeyringpwd2(ivadmin_context          ctx,
                              const char               *cfg_file_name,
                              int                      kdb_pwd_life,
                              ivadmin_response         *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
      ivadmin_cfg_setssltimeout2(ivadmin_context       ctx,
                                 const char            *cfg_file_name,
                                 int                   ssl_timeout,
                                 ivadmin_response      *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_cfg_setapplicationcert2(ivadmin_context     ctx,
                                   const char          *cfg_file_name,
                                   const char          *appl_cert,
                                   ivadmin_response    *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_cfg_addreplica2(ivadmin_context             ctx,
                           const char                  *cfg_file_name,
                           const char                  *ivacld_host,
                           int                         ivacld_port,
                           int                         ivacld_rank,
                           ivadmin_response            *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_cfg_chgreplica2(ivadmin_context             ctx,
                           const char                  *cfg_file_name,
                           const char                  *ivacld_host,
                           int                         ivacld_port,
                           int                         ivacld_rank,
                           ivadmin_response            *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_cfg_rmvreplica2(ivadmin_context             ctx,
                           const char                  *cfg_file_name,
                           const char                  *ivacld_host,
                           ivadmin_response            *rsp);


IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
    ivadmin_cfg_getvalue(ivadmin_context               ctx,
                         const char                    *cfg_file_name,
                         const char                    *stanza,
                         const char                    *key,
                         int                           *count,
                         char                          ***values,
                         ivadmin_response              *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
    ivadmin_cfg_removevalue(ivadmin_context            ctx,
                            const char                 *cfg_file_name,
                            const char                 *stanza,
                            const char                 *key,
                            int                        count,
                            char                       **values,
                            ivadmin_response           *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
    ivadmin_cfg_setvalue(ivadmin_context               ctx,
                         const char                    *cfg_file_name,
                         const char                    *stanza,
                         const char                    *key,
                         int                           count,
                         char                          **values,
                         int                           append,
                         int                           obfuscated,
                         ivadmin_response              *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
    ivadmin_cfg_setsvrpwd(ivadmin_context              ctx,
                          const char                   *cfg_file_name,
                          const char                   *newpassword,
                          ivadmin_response             *rsp);

/******************************************************
 *
 * Server APIs
 *
 ******************************************************/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_server_performtask(ivadmin_context ctx,
                              const char *server,
                              const char *task,
                              azn_attrlist_h_t *indata,
                              azn_attrlist_h_t *outdata,
                              unsigned long *resultcount,
                              char ***results,
                              ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_server_gettasklist(ivadmin_context ctx,
                              const char *server,
                              azn_attrlist_h_t *indata,
                              unsigned long *taskcount,
                              char ***tasks,
                              azn_attrlist_h_t *outdata,
                              unsigned long *resultcount,
                              char ***results,
                              ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
   ivadmin_server_replicate(ivadmin_context ctx,
                            const char *server,
                            ivadmin_response *rsp);


/******************************************************
 *
 * Authzrule APIs
 *
 ******************************************************/

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
    ivadmin_authzrule_create(
                ivadmin_context ctx,
                const char *ruleid,
                const char *ruledesc,
                const char *ruletext,
                const char *failreason,
                ivadmin_response *rsp);


IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
    ivadmin_authzrule_delete(
                ivadmin_context ctx,
                const char *ruleid,
                ivadmin_response *rsp);


IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
    ivadmin_authzrule_list(
                ivadmin_context ctx,
                unsigned long *count,
                char ***ruleids,
                ivadmin_response *rsp);


IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
    ivadmin_authzrule_get(
                ivadmin_context ctx,
                const char *ruleid,
                ivadmin_authzrule *rule,
                ivadmin_response *rsp);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
    ivadmin_authzrule_getid(
                ivadmin_authzrule rule);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
    ivadmin_authzrule_getdescription(
                ivadmin_authzrule rule);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
    ivadmin_authzrule_getruletext(
                ivadmin_authzrule rule);

IVADMIN_DECLSPEC const char * IVADMIN_CALLTYPE
    ivadmin_authzrule_getfailreason(
                ivadmin_authzrule rule);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
    ivadmin_authzrule_setdescription(
                ivadmin_context ctx,
                const char *ruleid,
                const char *description,
                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
    ivadmin_authzrule_setruletext(
                ivadmin_context ctx,
                const char *ruleid,
                const char *ruletext,
                ivadmin_response *rsp);

IVADMIN_DECLSPEC unsigned long IVADMIN_CALLTYPE
    ivadmin_authzrule_setfailreason(
                ivadmin_context ctx,
                const char *ruleid,
                const char *failreason,
                ivadmin_response *rsp);

#ifdef __cplusplus
    }
#endif

#endif /* IVADMINAPI_H */
