/*
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplicaion or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * Copyright (C) Tivoli Systems/IBM.  2000
 *
 * NAME
 *      pdlog.h - PolicyDirector Logging Service header
 *
 * DESCRIPTION
 *     This header file contains the definitions of Logging Service
 *     specific macros and the "C" API functions prototypes.
 */

#ifndef PD_LOG_H
#define PD_LOG_H

#include <ogauthzn.h>
#include <amlogtypes.h>

/*
 * Log Service API interfaces.
 */

#ifdef __cplusplus
extern "C" {
#endif

    AZN_EXPORT IVTstatus PDLogPoolOpen(char *config_file_p,
                                           char *stanza_p);

    AZN_EXPORT void PDLogPoolClose();

    AZN_EXPORT void *PDLogPool(char *category_p);

    AZN_EXPORT IVTbool PDLogPoolHasObservers(void *pool_h);

    AZN_EXPORT void *PDLogConfigureAgent(char *category_p,
                                             char *adaptor_p);

    AZN_EXPORT void PDLogDestroyAgent(void **agent_h);

    AZN_EXPORT void *PDLogAttachSink(char *poolname_p);

    AZN_EXPORT void *PDLogAttachSink1(char *poolname_p,
                                          const char *encoding); /* PD_LOG_ENCODING_UTF8 if data passed as UTF-8 */

    AZN_EXPORT void PDLogDetachSink(void **sink_h);

    AZN_EXPORT void *PDLogEventStart(void *event_sink_h);

    AZN_EXPORT char *PDLogEventGetData(void *event_sink_h);

    AZN_EXPORT void PDLogEventAddText(void *event_p,
                        IVTuint32 size,
                        char *fmt, ...);

    AZN_EXPORT void PDLogEventAddData(void *event_p,
                        size_t size,
                        void *segment);

    AZN_EXPORT void PDLogEventPokeData(void *event_p,
                        IVTuint32 offset,
                        size_t size,
                        void *segment);

    AZN_EXPORT void PDLogEventTimestamp(void *event_p);

    AZN_EXPORT void PDLogEventDiscard(void **event_p);

    AZN_EXPORT void PDLogEventCommit(void **event_p,
                          PDLogEnumCommit options,
                          IVTstatus *);

    AZN_EXPORT void PDLogEvent(void *pool_h,
                          IVTuint32 size,
                          char *fmt, ...);
/*
 * DEPRECATED PD3.7 BACKWARD COMPATIBILITY FUNCTIONS
 */
    AZN_EXPORT void *PDLogInitialize(char *logname_p,
                                         char *blade_p,
                                         char *logpath_p,
                                         IVTuint32 flush_secs,
                                         IVTuint32 logsize,
                                         IVTbool isbuffered,
                                         IVTstatus *status_p);

    AZN_EXPORT void PDLogTerminate(void **log_h_p);

#ifdef __cplusplus
}
#endif

#endif /* PD_LOG_H */
